@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 23713 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2009-10-28 11:08:29 +0000 (Wed, 28 Oct 2009) $
  */
 
 #ifdef __GNUC__
@@ -556,7 +556,7 @@ void PMIMEInfo::ReadFrom(istream &strm)
     strm >> line;
     if (line.IsEmpty())
       break;
-    if (line[0] == ' ') 
+    if (line[0] == ' ' || line[0] == '\t') // RFC 2822 section 2.2.2 & 2.2.3
       lastLine += line;
     else {
       AddMIME(lastLine);
