@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22731 $
+ * $Revision: 23598 $
  * $Author: rjongbloed $
- * $Date: 2009-05-27 01:38:51 +0000 (Wed, 27 May 2009) $
+ * $Date: 2009-10-06 23:18:58 +0000 (Tue, 06 Oct 2009) $
  */
 
 #include <ptlib.h>
@@ -72,6 +72,18 @@ class CustomAllocator
     void Deallocate(Type * p, size_t n)
     { a.deallocate(p, n); }
 
+    Type * Reallocate(Type * p, size_t old_n, size_t new_n)
+    { 
+      if (new_n == old_n)
+        return p;
+
+      Type * np = (Type *)a.allocate(new_n);
+      memcpy(np, p, (new_n < old_n) ? new_n : old_n );
+      a.deallocate(p, old_n); 
+
+      return np;
+    }
+
     Allocator a;
 };
 
@@ -247,7 +259,6 @@ PAbstractArray::PAbstractArray(PINDEX elementSizeInBytes,
     theArray = NULL;
   else if (dynamicAllocation) {
     PINDEX sizebytes = elementSize*GetSize();
-    //theArray = (char *)malloc(sizebytes);
     theArray = PAbstractArray_allocator.Allocate(sizebytes);
     PAssert(theArray != NULL, POutOfMemory);
     memcpy(theArray, PAssertNULL(buffer), sizebytes);
@@ -281,7 +292,6 @@ void PAbstractArray::CloneContents(const PAbstractArray * array)
 {
   elementSize = array->elementSize;
   PINDEX sizebytes = elementSize*GetSize();
-  //char * newArray = (char *)malloc(sizebytes);
   char * newArray = PAbstractArray_allocator.Allocate(sizebytes);
   if (newArray == NULL)
     reference->size = 0;
@@ -379,7 +389,7 @@ PBoolean PAbstractArray::InternalSetSize(PINDEX newSize, PBoolean force)
     if (newsizebytes == 0)
       newArray = NULL;
     else {
-      if ((newArray = (char *)malloc(newsizebytes)) == NULL)
+      if ((newArray = PAbstractArray_allocator.Allocate(newsizebytes)) == NULL)
         return PFalse;
   
       if (theArray != NULL)
@@ -394,22 +404,22 @@ PBoolean PAbstractArray::InternalSetSize(PINDEX newSize, PBoolean force)
     if (theArray != NULL) {
       if (newsizebytes == 0) {
         if (allocatedDynamically)
-          free(theArray);
+          PAbstractArray_allocator.Deallocate(theArray, oldsizebytes);
         newArray = NULL;
       }
       else if (allocatedDynamically) {
-        if ((newArray = (char *)realloc(theArray, newsizebytes)) == NULL)
+        if ((newArray = (char *)PAbstractArray_allocator.Reallocate(theArray, oldsizebytes, newsizebytes)) == NULL)
           return PFalse;
       }
       else {
-        if ((newArray = (char *)malloc(newsizebytes)) == NULL)
+        if ((newArray = PAbstractArray_allocator.Allocate(newsizebytes)) == NULL)
           return PFalse;
         memcpy(newArray, theArray, PMIN(newsizebytes, oldsizebytes));
         allocatedDynamically = PTrue;
       }
     }
     else if (newsizebytes != 0) {
-      if ((newArray = (char *)malloc(newsizebytes)) == NULL)
+      if ((newArray = PAbstractArray_allocator.Allocate(newsizebytes)) == NULL)
         return PFalse;
     }
     else
@@ -428,7 +438,7 @@ PBoolean PAbstractArray::InternalSetSize(PINDEX newSize, PBoolean force)
 void PAbstractArray::Attach(const void *buffer, PINDEX bufferSize)
 {
   if (allocatedDynamically && theArray != NULL)
-    free(theArray);
+    PAbstractArray_allocator.Deallocate(theArray, elementSize*GetSize());
 
   theArray = (char *)buffer;
   reference->size = bufferSize;
@@ -1925,11 +1935,13 @@ PString & PString::vsprintf(const char * fmt, va_list arg)
   PAssert(SetSize(2000), POutOfMemory);
   ::vsprintf(theArray+len, fmt, arg);
 #else
-  PINDEX size = len;
+  PINDEX providedSpace = 0;
+  int requiredSpace;
   do {
-    size += 1000;
-    PAssert(SetSize(size), POutOfMemory);
-  } while (_vsnprintf(theArray+len, size-len, fmt, arg) == -1);
+    providedSpace += 1000;
+    PAssert(SetSize(providedSpace+len), POutOfMemory);
+    requiredSpace = _vsnprintf(theArray+len, providedSpace, fmt, arg);
+  } while (requiredSpace == -1 || requiredSpace >= providedSpace);
 #endif // P_VXWORKS
 
   PAssert(MakeMinimumSize(), POutOfMemory);
