@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22552 $
- * $Author: csoutheren $
- * $Date: 2009-05-07 08:53:20 +0000 (Thu, 07 May 2009) $
+ * $Revision: 23518 $
+ * $Author: rjongbloed $
+ * $Date: 2009-09-23 06:52:27 +0000 (Wed, 23 Sep 2009) $
  */
 
 #ifdef __GNUC__
@@ -676,9 +676,10 @@ void PSafePtrBase::ExitSafetyMode(ExitSafetyModeOption ref)
   }
 
   if (ref == WithDereference && currentObject->SafeDereference()) {
-    PTRACE(6, "SafeColl\tDeleting object ("<<(void *)currentObject<<")");
-    delete currentObject;
+    PSafeObject * objectToDelete = currentObject;
     currentObject = NULL;
+    PTRACE(6, "SafeColl\tDeleting object (" << objectToDelete << ')');
+    delete objectToDelete;
   }
 }
 
