@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22356 $
+ * $Revision: 23336 $
  * $Author: rjongbloed $
- * $Date: 2009-03-31 23:57:19 +0000 (Tue, 31 Mar 2009) $
+ * $Date: 2009-09-05 01:02:14 +0000 (Sat, 05 Sep 2009) $
  */
 
 #define _OSUTIL_CXX
@@ -400,7 +400,7 @@ PString PX_GetThreadName(pthread_t id)
 {
   if (PProcessInstance != NULL) {
     PWaitAndSignal m(PProcessInstance->activeThreadMutex);
-    PThread & thread = PProcessInstance->activeThreads[(uintptr_t)id];
+    PThread & thread = PProcessInstance->activeThreads[_hptr(id)];
     return thread.GetThreadName();
   }
   return psprintf("%08x", id);
