@@ -26,13 +26,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22464 $
- * $Author: csoutheren $
- * $Date: 2009-04-24 18:23:36 +0000 (Fri, 24 Apr 2009) $
+ * $Revision: 23827 $
+ * $Author: rjongbloed $
+ * $Date: 2009-12-01 22:39:07 +0000 (Tue, 01 Dec 2009) $
  */
 
 #include <ptlib/socket.h>
-#include <sched.h>  // for sched_yield
+#include <sched.h>
 #include <pthread.h>
 #include <sys/resource.h>
 
@@ -97,6 +97,50 @@ static PBoolean PAssertThreadOp(int retval,
 }
 
 
+#if defined(P_LINUX)
+static int GetSchedParam(PThread::Priority priority, sched_param & param)
+{
+  /*
+    Set realtime scheduling if our effective user id is root (only then is this
+    allowed) AND our priority is Highest.
+      I don't know if other UNIX OSs have SCHED_FIFO and SCHED_RR as well.
+
+    WARNING: a misbehaving thread (one that never blocks) started with Highest
+    priority can hang the entire machine. That is why root permission is 
+    neccessary.
+  */
+
+  memset(&param, 0, sizeof(sched_param));
+
+  switch (priority) {
+    case PThread::HighestPriority :
+      param.sched_priority = sched_get_priority_max(SCHED_RR);
+      break;
+
+    case PThread::HighPriority :
+      param.sched_priority = sched_get_priority_min(SCHED_RR);
+      break;
+
+#ifdef SCHED_BATCH
+    case PThread::LowestPriority :
+    case PThread::LowPriority :
+      return SCHED_BATCH;
+#endif
+
+    default : // PThread::NormalPriority :
+      return SCHED_OTHER;
+  }
+
+  if (geteuid() == 0)
+    return SCHED_RR;
+
+  param.sched_priority = 0;
+  PTRACE(2, "PTLib\tNo permission to set priority level " << priority);
+  return SCHED_OTHER;
+}
+#endif
+
+
 PDECLARE_CLASS(PHouseKeepingThread, PThread)
   public:
     PHouseKeepingThread()
@@ -250,7 +294,7 @@ PBoolean PProcess::PThreadKill(pthread_t id, unsigned sig)
 {
   PWaitAndSignal m(activeThreadMutex);
 
-  if (!activeThreads.Contains((unsigned)id)) 
+  if (!activeThreads.Contains(_hptr(id))) 
     return PFalse;
 
   return pthread_kill(id, sig) == 0;
@@ -259,8 +303,8 @@ PBoolean PProcess::PThreadKill(pthread_t id, unsigned sig)
 void PProcess::PXSetThread(pthread_t id, PThread * thread)
 {
   activeThreadMutex.Wait();
-  PThread * currentThread = activeThreads.GetAt((PINDEX)id);
-  activeThreads.SetAt((PINDEX)id, thread);
+  PThread * currentThread = activeThreads.GetAt(_hptr(id));
+  activeThreads.SetAt(_hptr(id), thread);
   activeThreadMutex.Signal();
 
   if (currentThread != NULL) 
@@ -386,14 +430,15 @@ PThread::~PThread()
     process.SignalTimerChange();
 
     // last gasp tracing
-    PTRACE(1, "PTLib\tDestroyed thread " << this << ' ' << threadName << "(id = " << ::hex << id << ::dec << ")");
+    PTRACE(5, "PTLib\tDestroyed thread " << this << ' ' << threadName << "(id = " << ::hex << id << ::dec << ")");
 
 
     // if thread was started, remove it from the active thread list and detach it to release thread resources
     if (id != 0) {
       process.activeThreadMutex.Wait();
-      pthread_detach(id);
-      process.activeThreads.SetAt((unsigned)id, NULL);
+      if (PX_origStackSize != 0)
+        pthread_detach(id);
+      process.activeThreads.SetAt(_hptr(id), NULL);
       process.activeThreadMutex.Signal();
     }
 
@@ -477,19 +522,10 @@ void PThread::Restart()
   // Set a decent (256K) stack size that won't eat all virtual memory
   pthread_attr_setstacksize(&threadAttr, 16*PTHREAD_STACK_MIN);
 
-  /*
-    Set realtime scheduling if our effective user id is root (only then is this
-    allowed) AND our priority is Highest.
-      As far as I can see, we could use either SCHED_FIFO or SCHED_RR here, it
-    doesn't matter.
-      I don't know if other UNIX OSs have SCHED_FIFO and SCHED_RR as well.
+  struct sched_param sched_params;
+  PAssertPTHREAD(pthread_attr_setschedpolicy, (&threadAttr, GetSchedParam(PX_priority, sched_params)));
+  PAssertPTHREAD(pthread_attr_setschedparam,  (&threadAttr, &sched_params));
 
-    WARNING: a misbehaving thread (one that never blocks) started with Highest
-    priority can hang the entire machine. That is why root permission is 
-    neccessary.
-  */
-  if ((geteuid() == 0) && (PX_priority == HighestPriority))
-    PAssertPTHREAD(pthread_attr_setschedpolicy, (&threadAttr, SCHED_FIFO));
 #elif defined(P_RTEMS)
   pthread_attr_setstacksize(&threadAttr, 2*PTHREAD_MINIMUM_STACK_SIZE);
   pthread_attr_setinheritsched(&threadAttr, PTHREAD_EXPLICIT_SCHED);
@@ -681,29 +717,14 @@ void PThread::SetPriority(Priority priorityLevel)
 {
   PX_priority = priorityLevel;
 
+  if (IsTerminated())
+    return;
+
 #if defined(P_LINUX)
-  if (IsTerminated())
-    return;
-
-  struct sched_param sched_param;
-  
-  if ((priorityLevel == HighestPriority) && (geteuid() == 0) ) {
-    sched_param.sched_priority = sched_get_priority_min( SCHED_FIFO );
-    
-    PAssertPTHREAD(pthread_setschedparam, (PX_threadId, SCHED_FIFO, &sched_param));
-  }
-  else if (priorityLevel != HighestPriority) {
-    /* priority 0 is the only permitted value for the SCHED_OTHER scheduler */ 
-    sched_param.sched_priority = 0;
-    
-    PAssertPTHREAD(pthread_setschedparam, (PX_threadId, SCHED_OTHER, &sched_param));
-  }
-#endif
-
-#if defined(P_MACOSX)
-  if (IsTerminated())
-    return;
+  struct sched_param params;
+  PAssertPTHREAD(pthread_setschedparam, (PX_threadId, GetSchedParam(priorityLevel, params), &params));
 
+#elif defined(P_MACOSX)
   if (priorityLevel == HighestPriority) {
     /* get fixed priority */
     {
@@ -728,7 +749,7 @@ void PThread::SetPriority(Priority priorityLevel)
       // spawning thread's priority
       
       relativePriority = 62 - GetThreadBasePriority();
-      PTRACE(1,  "relativePriority is " << relativePriority << " base priority is " << GetThreadBasePriority());
+      PTRACE(3,  "relativePriority is " << relativePriority << " base priority is " << GetThreadBasePriority());
       
       thePrecedencePolicy.importance = relativePriority;
       result = thread_policy_set (pthread_mach_thread_np(PX_threadId),
@@ -747,23 +768,28 @@ void PThread::SetPriority(Priority priorityLevel)
 PThread::Priority PThread::GetPriority() const
 {
 #if defined(LINUX)
-  int schedulingPolicy;
-  struct sched_param schedParams;
+  int policy;
+  struct sched_param params;
   
-  PAssertPTHREAD(pthread_getschedparam, (PX_threadId, &schedulingPolicy, &schedParams));
+  PAssertPTHREAD(pthread_getschedparam, (PX_threadId, &policy, &params));
   
-  switch( schedulingPolicy )
+  switch (policy)
   {
     case SCHED_OTHER:
       break;
-      
+
     case SCHED_FIFO:
     case SCHED_RR:
-      return HighestPriority;
-      
+      return params.sched_priority > sched_get_priority_min(policy) ? HighestPriority : HighPriority;
+
+#ifdef SCHED_BATCH
+    case SCHED_BATCH :
+      return LowPriority;
+#endif
+
     default:
       /* Unknown scheduler. We don't know what priority this thread has. */
-      PTRACE(1, "PTLib\tPThread::GetPriority: unknown scheduling policy #" << schedulingPolicy);
+      PTRACE(1, "PTLib\tPThread::GetPriority: unknown scheduling policy #" << policy);
   }
 #endif
 
@@ -884,7 +910,7 @@ void PThread::Terminate()
 PBoolean PThread::IsTerminated() const
 {
   pthread_t id = PX_threadId;
-  return (id == 0) || (pthread_kill(id, 0) != 0);
+  return (id == 0) || (PX_origStackSize <= 0) || (pthread_kill(id, 0) != 0);
 }
 
 
@@ -910,7 +936,7 @@ void PThread::WaitForTermination() const
                // CPU usage and also yeilds so other threads can run.
   }
 
-  PTRACE(2, "WaitForTermination on " << id << " finished");
+  PTRACE(5, "WaitForTermination on " << id << " finished");
 }
 
 
